@@ -2,10 +2,11 @@ import sys
 import types
 import os
 
-__version__ = '1.10.0'
+__version__ = '1.11.0'
 
 # set Python env variable to keep track of example data dir
-DATADIR = os.path.join(sys.prefix, 'orbitize_example_data')
+orbitize_dir = os.path.dirname(__file__)
+DATADIR = os.path.join(orbitize_dir, 'example_data/')
 
 # define functions for pickling class methods
 def _pickle_method(method):
