@@ -70,8 +70,8 @@ class Sampler(abc.ABC):
         errs = np.array([self.system.data_table['quant1_err'],
                          self.system.data_table['quant2_err']]).T
 
-        # TODO: THIS ONLY WORKS FOR 1 PLANET. Make this a for loop to work for multiple planets.
-        seppa_indices = np.union1d(self.system.seppa[0], self.system.seppa[1])
+        # grab all seppa indices
+        seppa_indices = self.system.all_seppa
 
         # compute lnlike
         lnlikes = self.lnlike(data, errs, model, jitter, seppa_indices)
@@ -109,10 +109,14 @@ class OFTI(Sampler,):
         self.priors = self.system.sys_priors
 
         # convert RA/Dec rows to sep/PA
-        body_num = 1  # the first planet; MODIFY THIS LATER FOR MULTIPLE PLANETS
-        if len(self.system.radec[body_num]) > 0:
-            print('Converting ra/dec data points in data_table to sep/pa. Original data are stored in input_table.')
-            self.system.convert_data_table_radec2seppa(body_num=body_num)
+        convert_warning_print = False
+        for body_num in np.arange(self.system.num_secondary_bodies) + 1:
+            if len(self.system.radec[body_num]) > 0:
+                # only print the warning once. 
+                if not convert_warning_print:
+                    print('Converting ra/dec data points in data_table to sep/pa. Original data are stored in input_table.')
+                    convert_warning_print = True
+                self.system.convert_data_table_radec2seppa(body_num=body_num)
 
         # these are of type astropy.table.column
         self.sep_observed = self.system.data_table[np.where(
@@ -123,6 +127,8 @@ class OFTI(Sampler,):
             self.system.data_table['quant_type'] == 'seppa')]['quant1_err'].copy()
         self.pa_err = self.system.data_table[np.where(
             self.system.data_table['quant_type'] == 'seppa')]['quant2_err'].copy()
+        self.meas_object = self.system.data_table[np.where(
+            self.system.data_table['quant_type'] == 'seppa')]['object'].copy()
 
         # this is OK, ONLY IF we are only using self.epochs for computing RA/Dec from Keplerian elements
         self.epochs = np.array(self.system.data_table['epoch']) - self.system.tau_ref_epoch
@@ -135,8 +141,22 @@ class OFTI(Sampler,):
             self.system.data_table['quant_type'] == 'rv')]['epoch']) - self.system.tau_ref_epoch
 
         # choose scale-and-rotate epoch
-        self.epoch_idx = np.argmin(self.sep_err)  # epoch with smallest error
-
+        # for multiplanet support, this is now a list. 
+        # For each planet, we find the measurment for it that corresponds to the smallest astrometric error
+        self.epoch_idx = []
+        min_sep_indices = np.argsort(self.sep_err) # indices of sep err sorted from smallest to higheset
+        min_sep_indices_body = self.meas_object[min_sep_indices] # the corresponding body_num that these sorted measurements correspond to
+        for i in range(self.system.num_secondary_bodies):
+            body_num = i + 1
+            this_object_meas = np.where(min_sep_indices_body == body_num)[0]
+            if np.size(this_object_meas) == 0:
+                # no data, no scaling
+                self.epoch_idx.append(None)
+                continue
+            # get the smallest measurement belonging to this body
+            best_epoch = min_sep_indices[this_object_meas][0] # already sorted by argsort
+            self.epoch_idx.append(best_epoch)
+        
         if len(self.system.rv[0]) > 0 and self.system.fit_secondary_mass:  # checking for RV data
             self.rv_observed = self.system.data_table[np.where(
                 self.system.data_table['quant_type'] == 'rv')]['quant1'].copy()
@@ -151,7 +171,8 @@ class OFTI(Sampler,):
             sampler_name=self.__class__.__name__,
             post=None,
             lnlike=None,
-            tau_ref_epoch=self.system.tau_ref_epoch
+            tau_ref_epoch=self.system.tau_ref_epoch,
+            num_secondary_bodies=self.system.num_secondary_bodies
         )
 
     def prepare_samples(self, num_samples):
@@ -178,63 +199,75 @@ class OFTI(Sampler,):
             else: # param is fixed & has no prior
                 samples[i, :] = self.priors[i] * np.ones(num_samples)
 
-        # sma, ecc, inc, argp, lan, tau, plx, mtot = [s for s in samples]
-        sma = samples[0,:]
-        ecc = samples[1,:]
-        inc = samples[2,:]
-        argp = samples[3,:]
-        lan = samples[4,:]
-        tau = samples[5,:]
-        plx = samples[6,:]
-        if self.system.fit_secondary_mass:
-            m0 = samples[-1,:]
-            m1 = samples[-2,:]
-            mtot = m0 + m1
-        else:
-            mtot = samples[-1,:]
-            m1 = None
+        for body_num in np.arange(self.system.num_secondary_bodies):
+            # sma, ecc, inc, argp, lan, tau, plx, mtot = [s for s in samples]
+            ref_ind = 6 * body_num
+            sma = samples[ref_ind,:]
+            ecc = samples[ref_ind + 1,:]
+            inc = samples[ref_ind + 2,:]
+            argp = samples[ref_ind + 3,:]
+            lan = samples[ref_ind + 4,:]
+            tau = samples[ref_ind + 5,:]
+            plx = samples[6 * self.system.num_secondary_bodies,:]
+            if self.system.fit_secondary_mass:
+                m0 = samples[-1,:]
+                m1 = samples[-1-self.system.num_secondary_bodies+body_num,:]
+                mtot = m0 + m1
+            else:
+                mtot = samples[-1,:]
+                m1 = None
+            
+            if "gamma" in self.system.labels:
+                gamma = samples[6 * self.system.num_secondary_bodies + 1, :]  # Rob: added gamma and sigma parameters
+            if "sigma" in self.system.labels:
+                sigma = samples[6 * self.system.num_secondary_bodies + 2, :]
 
-        period_prescale = np.sqrt(
-            4*np.pi**2*(sma*u.AU)**3/(consts.G*(mtot*u.Msun))
-        )
-        period_prescale = period_prescale.to(u.day).value
-        meananno = self.epochs[self.epoch_idx]/period_prescale - tau
+            min_epoch = self.epoch_idx[body_num]
+            if min_epoch is None:
+                # Don't need to rotate and scale if no astrometric measurments for this body. Brute force rejection sampling
+                continue
 
-        # compute sep/PA of generated orbits
-        ra, dec, vc = orbitize.kepler.calc_orbit(
-            self.epochs[self.epoch_idx], sma, ecc, inc, argp, lan, tau, plx, mtot, 
-            mass_for_Kamp=m1
-        )
-        sep, pa = orbitize.system.radec2seppa(ra, dec) # sep[mas], PA[deg]
+            period_prescale = np.sqrt(
+                4*np.pi**2*(sma*u.AU)**3/(consts.G*(mtot*u.Msun))
+            )
+            period_prescale = period_prescale.to(u.day).value
+            meananno = self.epochs[min_epoch]/period_prescale - tau
 
-        # generate Gaussian offsets from observational uncertainties
-        sep_offset = np.random.normal(
-            0, self.sep_err[self.epoch_idx], size=num_samples
-        )
-        pa_offset =  np.random.normal(
-            0, self.pa_err[self.epoch_idx], size=num_samples
-        )
+            # compute sep/PA of generated orbits
+            ra, dec, vc = orbitize.kepler.calc_orbit(
+                self.epochs[min_epoch], sma, ecc, inc, argp, lan, tau, plx, mtot, 
+                mass_for_Kamp=m1
+            )
+            sep, pa = orbitize.system.radec2seppa(ra, dec) # sep[mas], PA[deg]
 
-        # calculate correction factors
-        sma_corr = (sep_offset + self.sep_observed[self.epoch_idx])/sep
-        lan_corr = (pa_offset + self.pa_observed[self.epoch_idx] - pa)
+            # generate Gaussian offsets from observational uncertainties
+            sep_offset = np.random.normal(
+                0, self.sep_err[min_epoch], size=num_samples
+            )
+            pa_offset =  np.random.normal(
+                0, self.pa_err[min_epoch], size=num_samples
+            )
 
-        # perform scale-and-rotate
-        sma *= sma_corr # [AU]
-        lan += np.radians(lan_corr) # [rad]
-        lan = lan % (2*np.pi)
+            # calculate correction factors
+            sma_corr = (sep_offset + self.sep_observed[min_epoch])/sep
+            lan_corr = (pa_offset + self.pa_observed[min_epoch] - pa)
 
-        period_new = np.sqrt(
-            4*np.pi**2*(sma*u.AU)**3/(consts.G*(mtot*u.Msun))
-        )
-        period_new = period_new.to(u.day).value
+            # perform scale-and-rotate
+            sma *= sma_corr # [AU]
+            lan += np.radians(lan_corr) # [rad]
+            lan = lan % (2*np.pi)
 
-        tau = (self.epochs[self.epoch_idx]/period_new - meananno) % 1
+            period_new = np.sqrt(
+                4*np.pi**2*(sma*u.AU)**3/(consts.G*(mtot*u.Msun))
+            )
+            period_new = period_new.to(u.day).value
 
-        # updates samples with new values of sma, pan, tau
-        samples[0,:] = sma
-        samples[4,:] = lan
-        samples[5,:] = tau
+            tau = (self.epochs[min_epoch]/period_new - meananno) % 1
+
+            # updates samples with new values of sma, pan, tau
+            samples[ref_ind,:] = sma
+            samples[ref_ind + 4,:] = lan
+            samples[ref_ind + 5,:] = tau
 
         return samples
 
@@ -472,7 +505,8 @@ class MCMC(Sampler):
             sampler_name=self.__class__.__name__,
             post=None,
             lnlike=None,
-            tau_ref_epoch=system.tau_ref_epoch
+            tau_ref_epoch=system.tau_ref_epoch,
+            num_secondary_bodies=system.num_secondary_bodies
         )
 
         if self.num_temps > 1:
@@ -610,13 +644,15 @@ class MCMC(Sampler):
                 self.num_walkers, self.num_params, self._logl,
                 threads=self.num_threads, kwargs={'include_logp': True}
             )
-
-        for pos, lnprob, lnlike in sampler.sample(self.curr_pos, iterations=burn_steps, thin=thin):
+        
+        # we're using args because emcee < 3.0 has three return values whereas emcee > 3.0 has
+        # four. We can explicitly declare 4 variables instead of args in the future. 
+        for args in sampler.sample(self.curr_pos, iterations=burn_steps, thin=thin):
             pass
 
         sampler.reset()
         try:
-            self.curr_pos = pos
+            self.curr_pos = args[0]
         except UnboundLocalError:  # 0 step burn-in (pos is not defined)
             pass
         print('Burn in complete')
@@ -625,15 +661,17 @@ class MCMC(Sampler):
 
         assert (nsteps > 0), 'Total_orbits must be greater than num_walkers.'
 
-        i = 0
-        for pos, lnprob, lnlike in sampler.sample(p0=self.curr_pos, iterations=nsteps, thin=thin):
-            i += 1
+        i=0
+        # we're using args because emcee < 3.0 has three return values whereas emcee > 3.0 has
+        # four. We can explicitly declare 4 variables instead of args in the future. 
+        for args in sampler.sample(self.curr_pos, iterations=nsteps, thin=thin):
+            i+=1
             # print progress statement
             if i % 5 == 0:
                 print(str(i)+'/'+str(nsteps)+' steps completed', end='\r')
         print('')
 
-        self.curr_pos = pos
+        self.curr_pos = args[0] # note that args[0] is pos output
 
         # TODO: Need something here to pick out temperatures, just using lowest one for now
         self.chain = sampler.chain
@@ -776,7 +814,8 @@ class MCMC(Sampler):
             post=flat_chopped_chain,
             lnlike=flat_chopped_lnlikes,
             tau_ref_epoch=self.system.tau_ref_epoch,
-            labels=self.system.labels
+            labels=self.system.labels,
+            num_secondary_bodies=self.system.num_secondary_bodies
         )
 
         # Print a confirmation
